close all;
clear all;
clc;

%%%%%%%%%%%%%%%%%%%
% Global Variable %
global S;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add release Directory to Matlab Path %
S.MexFilePath = ['..',filesep,'x64',filesep,'Release'];
addpath(S.MexFilePath);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Initial Computing Type Setup %%%
% 'CUDA ML'
% 'CPU ML'
% 'CPU FFT'
% 'DBZP'
ComputingType = 'DBZP';
S.A = 1;                  % Base GPS Signal Amplitude
S.SwiftStep = 40;         % Number of Steps per CUDA kernel (ML)
S.MsgTitle = 'Warning';   % Default Warning Message Title
S.NumLabels = 9;          % Number of 3D Plot Labels
S.Lw = 1;                 % Linewidth of 2D Slices
S.GS = 0.15;              % Normalized Spacing for Panels and Plots

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% Create Main Figure %%%%%%%
S.ScreenSize = get(0,'ScreenSize');
S.ScW = S.ScreenSize(3);
S.ScH = S.ScreenSize(4);
S.FWidth  = 900;  % Figure Width
S.FHeight = 640;  % Figure Height
S.FminW   = 900;  % Minimum Figure Width
S.FminH   = 640;  % Minimum Figure Height

S.fm = figure;
S.fm.Position = [(S.ScW - S.FWidth)/2, (S.ScH - S.FHeight)/2, S.FWidth S.FHeight];
S.fm.Name = 'GPS L1 Signal Processing';
S.fm.NumberTitle = 'off';
S.fm.Color = 'white';
S.fm.Tag = 'MainFigure';
S.fm.SizeChangedFcn   = @Callback_Resize_Components;
S.fm.ResizeFcn        = @Callback_Resize_Components;
S.fm.CloseRequestFcn  = @Callback_Close_Figures;
S.fm.MenuBar = 'none';
S.fm.ToolBar = 'figure';
warning('off','MATLAB:HandleGraphics:ObsoletedProperty:JavaFrame');
javaFrame = get(S.fm,'JavaFrame');
javaFrame.setFigureIcon(javax.swing.ImageIcon('nvidia.PNG'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Create Information Figure %%%%%%%
S.fi = figure('name','ApplicationInfo','Visible','off');
S.fi.NumberTitle = 'off';
S.fi.Color = 'white';
S.fi.Tag = 'InfoFigure';
S.fi.ToolBar = 'none';
S.fi.MenuBar = 'none';
S.fi.Resize = 'off';
S.fi.CloseRequestFcn = @Callback_Info_Hide;
javaFrame = get(S.fi,'JavaFrame');
javaFrame.setFigureIcon(javax.swing.ImageIcon('nvidia.PNG'));
Ori = S.fi.Position;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% User Interface Constants %%%%%%%
% (This is not the most efficient way to create a UI)
S.VertSpacing = 30;                             % Vertical Spacing of Components
S.RadioStart = S.FHeight - S.VertSpacing - 10;  % Start of Components
S.RadioWidth = 150;                             % Radio Button Width
S.RadioHeight = 30;                             % Radio Button Height
S.EditWidth = 50;                               % Edit box Width
S.EditHeight = 20;                              % Edit Box Height
S.LabWidth = 130;                               % Label Width
S.LabHeight = 20;                               % Label Height
S.EditHOffset = S.LabWidth + 10;                % Horizontal Offset for EditBoxes
S.GeneralHOffset = 10;                          % Horizontal Offset for all components
S.AxSpacing = 20;                               % Spacing for plot Axes
S.ComponentsWidth = S.EditWidth + S.LabWidth + 2*S.GeneralHOffset + 10 + 10;
S.ComponentsHeight = 15*S.VertSpacing + 10 + 10;
S.AxWidth  = (S.FWidth - S.ComponentsWidth - 3*S.AxSpacing)/2;
S.AxHeight = (S.FHeight - 3*S.AxSpacing)/2;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Assign Information Text to the Corresponding Figure %%%%%%%%
fid = fopen('Info.txt');
InfoText = [];
textline = fgets(fid);
while ischar(textline)
    InfoText = [InfoText,textline];
    textline = fgets(fid);
end
fclose(fid);

SimText = uicontrol(S.fi,'Style','text',...
    'Position',[0,0,Ori(3),Ori(4)],...
    'Tag','KeepItSimple',...
    'String',InfoText,...
    'HandleVisibility','on',...
    'HorizontalAlignment','Left');
set(0, 'currentfigure', S.fm);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% Create Radio Buttons %%%%%%%%
S.ButtonGroup1 = uibuttongroup();
S.ButtonGroup1.Position = [0 0 S.ComponentsWidth/S.FWidth 1];
S.ButtonGroup1.Visible = 'on';
S.ButtonGroup1.Tag = 'ButtnGroup1';
S.ButtonGroup1.Title = 'Configuration';
S.ButtonGroup1.TitlePosition = 'CenterTop';

VertC = 1;
S.Radio1 = uicontrol(S.ButtonGroup1,'Style','radiobutton',...
                  'String','CUDA ML (Fast)',...
                  'HandleVisibility','on','Tag','Radio1',...
                  'Callback',@Callback_Radio);
S.Radio2 = uicontrol(S.ButtonGroup1,'Style','radiobutton','String','CPU ML (Slow)',...
                  'HandleVisibility','on','Tag','Radio2',...
                  'Callback',@Callback_Radio); VertC = VertC + 1;
S.Radio3 = uicontrol(S.ButtonGroup1,'Style','radiobutton','String','CPU FFTCor (Fast)',...
                  'HandleVisibility','on','Tag','Radio3',...
                  'Callback',@Callback_Radio); VertC = VertC + 1;
S.Radio4 = uicontrol(S.ButtonGroup1,'Style','radiobutton',...
                  'String','DBZP (Fast)',...
                  'HandleVisibility','on','Tag','Radio4',...
                  'Callback',@Callback_Radio); VertC = VertC + 1;
VertL = VertC;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%      
%%%%%%%% Create Text Edit Boxes %%%%%%%%
S.EditDelay = uicontrol(S.ButtonGroup1,'Style','edit','String','100',...
    'HandleVisibility','on','Tag','EditDelay',...
    'HorizontalAlignment','Center');VertC = VertC + 1;
S.EditPhase = uicontrol(S.ButtonGroup1,'Style','edit','String','117',...
    'HandleVisibility','on','Tag','EditPhase',...
    'HorizontalAlignment','Center');VertC = VertC + 1;
S.EditOffset = uicontrol(S.ButtonGroup1,'Style','edit','String','1000',...
    'HandleVisibility','on','Tag','EditOffset',...
    'HorizontalAlignment','Center');VertC = VertC + 1;
S.EditSweep = uicontrol(S.ButtonGroup1,'Style','edit','String','500',...
    'HandleVisibility','on','Tag','EditSweep',...
    'HorizontalAlignment','Center');VertC = VertC + 1;
S.EditFStart = uicontrol(S.ButtonGroup1,'Style','edit','String','-6000',...
    'HandleVisibility','on','Tag','EditFStart',...
    'HorizontalAlignment','Center');VertC = VertC + 1;
S.EditFStop = uicontrol(S.ButtonGroup1,'Style','edit','String','6000',...
    'HandleVisibility','on','Tag','EditFStop',...
    'HorizontalAlignment','Center');VertC = VertC + 1;
S.EditSNR = uicontrol(S.ButtonGroup1,'Style','edit','String','20',...
    'HandleVisibility','on','Tag','EditSNR',...
    'HorizontalAlignment','Center');VertC = VertC + 1;
S.EditGPSDec = uicontrol(S.ButtonGroup1,'Style','edit','String','13',...
    'HandleVisibility','on','Tag','EditGPSDec',...
    'HorizontalAlignment','Center');VertC = VertC + 1;
S.EditGPSCod = uicontrol(S.ButtonGroup1,'Style','edit','String','13',...
    'HandleVisibility','on','Tag','EditGPSCod',...
    'HorizontalAlignment','Center');VertC = VertC + 1;
S.EditFsa = uicontrol(S.ButtonGroup1,'Style','edit','String','2000000',...
    'HandleVisibility','on','Tag','EditFsa',...
    'HorizontalAlignment','Center');VertC = VertC + 1;
S.EditTime = uicontrol(S.ButtonGroup1,'Style','edit','String','0.001',...
    'HandleVisibility','on','Tag','EditTime',...
    'HorizontalAlignment','Center');VertC = VertC + 1;
S.EditFPoints = uicontrol(S.ButtonGroup1,'Style','edit','String','200',...    
    'HandleVisibility','on','Tag','EditFpoints',...
    'HorizontalAlignment','Center',...
    'Visible','off');VertC = VertC + 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Generate Labels %%%%%%%
S.LabDelay = uicontrol(S.ButtonGroup1,'Style','text','String','Delay in Samples [-]',...
    'HandleVisibility','on','Tag','LabDelay',...
    'HorizontalAlignment','Left');VertL = VertL + 1;
S.LabPhase = uicontrol(S.ButtonGroup1,'Style','text','String','Initial Phase [Deg]',...
    'HandleVisibility','on','Tag','LabPhase',...
    'HorizontalAlignment','Left');VertL = VertL + 1;
S.LabOffset = uicontrol(S.ButtonGroup1,'Style','text','String','Doppler Freq [Hz]',...
    'HandleVisibility','on','Tag','LabOffset',...
    'HorizontalAlignment','Left');VertL = VertL + 1;
S.LabSweep = uicontrol(S.ButtonGroup1,'Style','text','String','Freq. Sweep Step [Hz]',...
    'HandleVisibility','on','Tag','LabSweep',...
    'HorizontalAlignment','Left');VertL = VertL + 1;
S.LabFStart = uicontrol(S.ButtonGroup1,'Style','text','String','Freg. Start [Hz]',...
    'HandleVisibility','on','Tag','LabFStart',...
    'HorizontalAlignment','Left');VertL = VertL + 1;
S.LabFStop = uicontrol(S.ButtonGroup1,'Style','text','String','Freq. Stop [Hz]',...
    'HandleVisibility','on','Tag','LabFStop',...
    'HorizontalAlignment','Left');VertL = VertL + 1;
S.LabSNR = uicontrol(S.ButtonGroup1,'Style','text','String','GPS Signal SNR [dB]',...
    'HandleVisibility','on','Tag','LabSNR',...
    'HorizontalAlignment','Left');VertL = VertL + 1;
S.LabGPSDec = uicontrol(S.ButtonGroup1,'Style','text','String','PRN Decoding index [1-37]',...
    'HandleVisibility','on','Tag','LabGPSDec',...
    'HorizontalAlignment','Left');VertL = VertL + 1;
S.LabGPSCod = uicontrol(S.ButtonGroup1,'Style','text','String','PRN Coding index [1-37]',...
    'HandleVisibility','on','Tag','LabGPSCod',...
    'HorizontalAlignment','Left');VertL = VertL + 1;
S.LabFsa = uicontrol(S.ButtonGroup1,'Style','text','String','Sampling Frequency [Hz]',...
    'HandleVisibility','on','Tag','LabFsa',...
    'HorizontalAlignment','Left');VertL = VertL + 1;
S.LabTime = uicontrol(S.ButtonGroup1,'Style','text','String','Signal time [s]',...
    'HandleVisibility','on','Tag','LabTime',...
    'HorizontalAlignment','Left');VertL = VertL + 1;
S.LabFPoints = uicontrol(S.ButtonGroup1,'Style','text','String','FFT Resolution [Samples]',...
    'HandleVisibility','on','Tag','LabFPoints',...
    'HorizontalAlignment','Left',...
    'Visible','off');VertL = VertL + 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Generate 3 Panels %%%%%%%%
S.PDop = uipanel(); % Doppler
S.PDel = uipanel(); % Delay
S.P3D  = uipanel(); % 3D Space

S.FreeWidth  = S.FWidth - S.ComponentsWidth - 3*S.AxSpacing;
S.FreeHeight = S.FHeight - 3*S.AxSpacing;

S.PDop.Parent = S.fm;
S.PDop.Position = [(S.ComponentsWidth + S.AxSpacing)/S.FWidth,(S.AxSpacing)/S.FHeight,(S.FreeWidth/2)/S.FWidth,(S.FreeHeight/2)/S.FHeight];
S.PDop.BackgroundColor = 'white';
S.PDop.Title = 'Doppler Cut';
S.PDop.Tag = 'DopplerPanel';
S.PDop.ForegroundColor = 'black';
S.PDop.FontSize = 12;

S.PDel.Parent = S.fm;
S.PDel.Position = [(S.ComponentsWidth + 2*S.AxSpacing + (S.FreeWidth/2))/S.FWidth,(S.AxSpacing)/S.FHeight,(S.FreeWidth/2)/S.FWidth,(S.FreeHeight/2)/S.FHeight];
S.PDel.BackgroundColor = 'white';
S.PDel.Title = 'Delay Cut';
S.PDel.Tag = 'DelayPanel';
S.PDel.ForegroundColor = 'black';
S.PDel.FontSize = 12;


S.P3D.Parent = S.fm;
S.P3D.Position = [(S.ComponentsWidth + S.AxSpacing)/S.FWidth,(2*S.AxSpacing + (S.FreeHeight/2))/S.FHeight,(S.FreeWidth + S.AxSpacing)/S.FWidth,(S.FreeHeight/2)/S.FHeight];
S.P3D.BackgroundColor = 'white';
S.P3D.Title = 'GPS Doppler-Delay Space';
S.P3D.Tag = 'GPSPanel';
S.P3D.ForegroundColor = 'black';
S.P3D.FontSize = 12;

%%%%%%%%%%%%%%%%%%%
% Generate 3 Axes %
S.AxDop = axes();
S.AxDel = axes();
S.Ax3D  = axes();

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Prefill Plots with Random Data % 
surf(S.Ax3D, peaks(40));
surf(S.AxDel,peaks(40));
surf(S.AxDop,peaks(40));

S.AxDop.Parent = S.PDop;
S.AxDop.Position = [S.GS,S.GS,1 - 2*S.GS,1 - 2*S.GS];
S.AxDop.Tag               = 'AxDop';
S.AxDop.XGrid             = 'On';
S.AxDop.YGrid             = 'On';
S.AxDop.Visible           = 'On';
S.AxDop.HandleVisibility  = 'On';
S.AxDop.Color             = 'white';
S.AxDop.Clipping          = 'On';
S.AxDop.ClippingStyle     = 'rectangle';

S.AxDel.Parent = S.PDel;
S.AxDel.Position = [S.GS,S.GS,1 - 2*S.GS,1 - 2*S.GS];
S.AxDel.Tag               = 'AxDel';
S.AxDel.XGrid             = 'On';
S.AxDel.YGrid             = 'On';
S.AxDel.Visible           = 'On';
S.AxDel.HandleVisibility  = 'on';
S.AxDel.Color             = 'white';
S.AxDel.Clipping          = 'On';
S.AxDel.ClippingStyle     = 'rectangle';

S.Ax3D.Parent = S.P3D;
S.Ax3D.Position = [S.GS,S.GS,1 - 2*S.GS,1 - 2*S.GS];
S.Ax3D.Tag                = 'Ax3D';
S.Ax3D.Visible            = 'On';
S.Ax3D.HandleVisibility   = 'on';
S.Ax3D.Color              = 'white';
S.Ax3D.Clipping           = 'On';
S.Ax3D.ClippingStyle      = '3dbox';
colorbar;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Should Solve Problems with 3D Rotation %
props = {'CameraViewAngle','DataAspectRatio','PlotBoxAspectRatio'};
set(S.AxDop,props,get(S.AxDop,props));
set(S.AxDel,props,get(S.AxDel,props));
set(S.Ax3D, props,get(S.Ax3D,props));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Create Push Button for Computing %%%%%
S.Push1 = uicontrol(S.ButtonGroup1,'Style','pushbutton',...
               'String','Calculate',...
               'HandleVisibility','on',...
               'Callback',@Callback_GNSS_Compute);
           
%%%%%%%%%%%%%%%%%%%%%%%%
% Update All Positions %
Callback_Resize_Components(S.fm,0);
           
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% Adjust Toolbar and get rid off separators %%%%%%%
delete(findall(S.fm,'ToolTipString','Open File'));
delete(findall(S.fm,'ToolTipString','Print Figure'));
delete(findall(S.fm,'ToolTipString','New Figure'));
delete(findall(S.fm,'ToolTipString','Hide Plot Tools'));
delete(findall(S.fm,'ToolTipString','Show Plot Tools and Dock Figure'));
delete(findall(S.fm,'ToolTipString','Link Plot'));
delete(findall(S.fm,'ToolTipString','Brush/Select Data'));
delete(findall(S.fm,'ToolTipString','Edit Plot'));
delete(findall(S.fm,'ToolTipString','Insert Legend'));
delete(findall(S.fm,'ToolTipString','Insert ColorBar'));
Tb = findall(S.fm,'tag','FigureToolBar');
Ty = findall(Tb,'Type','uitoggletool');
set(Ty,'Separator','off');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create Simple Infor Toolbar Icon %
[img,map] = imread('Infobox.gif');
ICData = ind2rgb(img,map);
itool = uipushtool('Parent',Tb,'TooltipString','About',...
    'ClickedCallBack',@Callback_Info_Show,...
    'Tag','Info Box','HandleVisibility','on',...
    'CData',ICData);

%%%%%%%%%%%%%%%%%%%%%%%
% Add Custom Colormap % 
colormap morgenstemning;

