clear all;
close all;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get Display Resolution %
Displ = groot();
ScW = Displ.ScreenSize(3);
ScH = Displ.ScreenSize(4);
FiW = 1024;
FiH = 720;
Cls = perms([0,102,204])./255;
Cl_Orange = 1;
Cl_Orchid = 2;
Cl_Green  = 3;
Cl_Violet = 4;
Cl_Tyrkys = 5;
Cl_Blue   = 6;
Lw = 2; 

PRNNumber = 13;     % PRN Number (1-37)
NTc = 100;          % Number of samples per chip
Np = NTc*1023;      % Number of samples per signal period 
T = 1;              % Number of periods - integration time in ms
fs = Np * 1000;     % Sampling Frequency
DelMul = 0.8;       % A specific delay for multipath in chips
C = 3e8;            % Speed of light
Doppler = 0;        % Doppler offset
SMRO = 4;           % Signal to Multipath Ratio of Amplitude
Amp = 1/sqrt(SMRO); % Amplitude Modification of Multipath Signal

%%%%%%%%%%%%%%%%%%%%
% Generate Signals %
si = sGPSGen(PRNNumber,Np,T,Doppler,0.0);
rp = sGPSGen(PRNNumber,Np,T,Doppler,0.5);
%se = sGPSGen(PRNNumber,Np,T,Doppler,0 + DelMul/1023);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adjust Multipath Signal Amplitude % 
%Ps = 1/length(si)*sum((abs(si)).^2);   % Power of a standard signal with Ampl == 1
%se = Amp * se;                         % Amplide of a requied multipath signal
%Pe = 1/length(se)*sum((abs(se)).^2);   % Power of the multipath signal

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute for All Delays %
Step = 0.01;
MaxDelay = 1.5;
DelayData = 0:Step:MaxDelay;
dx = [NTc NTc/4 NTc/10];

PosErrorData = zeros(length(dx),length(DelayData));
NegErrorData = zeros(length(dx),length(DelayData));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Loop  Across Different Early-Late Distances %
for j = 1:length(dx)
    d = dx(j); 
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    % For ALl Possible Delays %
    for i = 1:length(DelayData)

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Compute Multipath Signal %
        CurrentDelay = 0 + fix((DelayData(i)/1023*100000))/100000;
        se = Amp.*sGPSGen(PRNNumber,Np,T,Doppler,CurrentDelay);

        %%%%%%%%%%%%%%%%%%%
        % Positive Search %
        sx = si + se;                                                   % Multipath simulation
        Rcross = abs(ifft(fft(sx).*conj(fft(rp))));                     % Correllation

        %%%%%%%%%%%%%%%%%%%%%%
        % Compute Time Error %
        for k = Np/2-NTc:Np/2
            if Rcross(k) <= Rcross(k + d)
                Te = k - Np/2 + d/2;
            end
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%
        % Save Distance Error %
        PosErrorData(j,i) = Te * 1/fs * C;
        
        %%%%%%%%%%%%%%%%%%%
        % Negative Search %
        sx = si - se;                                                   % Multipath Simulation
        Rcross = abs(ifft(fft(sx).*conj(fft(rp))));                     % Correllation
        
        %%%%%%%%%%%%%%%%%%%%%%
        % Compute Time Error %
        for k = Np/2-NTc:Np/2
            if Rcross(k) <= Rcross(k + d)
                Te = k - Np/2 + d/2;
            end
        end

        %%%%%%%%%%%%%%%%%%%%%%%
        % Save Distance Error %
        NegErrorData(j,i) = Te * 1/fs * C;
        
    end
end

%%%%%%%%%%%%%%%%%%%%%%%
%%% Display Results %%%
figure('name','Maximum Error for given delays and EL-Distances','position',[ScW/2 - FiW/2,ScH/2 - FiH/2,FiW,FiH]);
legends = {};
handles = [];
for i = 1:length(dx)

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Smooth Data With Moving-Average %
    %Temp1 = smooth(PosErrorData(i,:),'moving'); % Requires additional Package
    %Temp2 = smooth(NegErrorData(i,:),'moving'); % Requires additional Package
    Order = 20;
    a = 1;
    b = 1/Order * ones(1,Order);
    Temp1 = filter(b,a,PosErrorData(i,:));
    Temp2 = filter(b,a,NegErrorData(i,:));


    handles(i) = plot(DelayData,Temp1,'color',Cls(i,:),'linewidth',Lw);
    hold on;grid on;
    plot(DelayData,Temp2,'color',Cls(i,:),'linewidth',Lw);
    hold on;grid on;
    
    legends{i} = ['EL Distance: ',num2str(dx(i)/NTc),' Chips'];

end
legend(handles,legends);
title('GPS Multipath Errors')
xlabel('Delay [chip]');
ylabel('Error [m]');













