clear all;
close all;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get Display Resolution %
Displ = groot();
ScW = Displ.ScreenSize(3);
ScH = Displ.ScreenSize(4);
FiW = 1024;
FiH = 720;
Cls = perms([0,102,204])./255;
Cl_Orange = 1;
Cl_Orchid = 2;
Cl_Green  = 3;
Cl_Violet = 4;
Cl_Tyrkys = 5;
Cl_Blue   = 6;
Lw = 2; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Is Approximately 11 Hours and 58 Minutes %
OrbitTime = 11 * 3600 + 58*60; % Orbit time of the satellite [s]
Alt = 20350e3;                 % Altitude of the satellite [m]
Re = 6378e3;                   % Earth Radius [m]
Freq_L1 = 1575.42e6;           % Carrier frequency [Hz] - L1
C = 3e8;                       % Speed of light [m/s]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate Tangetial Speed of the Satellite %
R = 2*pi*(Alt+Re);
Vc = R/OrbitTime;
disp(horzcat('Tangential speed: ',num2str(Vc),' m/s'));

dmax = sqrt((Re+Alt)^2 - Re^2);   % Rise/set of the satellite 
t = 0:1:OrbitTime/2;              % Time axis in seconds
t = t./max(t);                    % Normalize Time
x = (Re + Alt)*cos(t.*pi);          % x-Posittion of the satellite
y = (Re + Alt)*sin(t.*pi);          % y-Posittion of the satellite

%%%%%%%%%%%%%%%%%%%%%
% Receiver Position %
xp = 0;
yp = Re;


Distances = sqrt( (x - xp).^2 + (y - yp).^2); % Distance between the receiver and transmitter
Vx = (Distances(2) - Distances(1))/1;         % Maximum radial speed of the satellite during rise/set
Doppler = gradient(Distances)/C*Freq_L1;      % Calculate Doppler Frequency Offsets

%Speed = sqrt( (x(2) - x(1))^2 + (y(2) - y(1))^2); % Just Checking
%disp(horzcat('Satellite speed: ',num2str(Speed),' m/s'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Display Results %%%%%%%
figure('name','Receiver-Transmitter Distance','position',[ScW/2 - FiW/2,ScH/2 - FiH/2,FiW,FiH]);
subplot(2,1,1);
plot(x./1000,Distances./1000,'color',Cls(Cl_Orchid,:),'linewidth',Lw);
xlabel('X distance in Earth-plane      [km]');
ylabel('Absolute distance to Satellite [km]');
grid on;

subplot(2,1,2);
plot(x./1000,Doppler,'color',Cls(Cl_Green,:),'linewidth',Lw);
ylabel('Doppler [Hz]');
xlabel('X distance in Earth-plane [km]');
grid on;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Custom Doppler Calculation %
Doppler = abs(Vx/C*Freq_L1);
fp = (C+0)/(C+Vx)*Freq_L1;
fp = fp/1e6;
disp(horzcat('Radial speed: ',num2str(abs(Vx)),' m/s'));
disp(horzcat('Doppler frequency offset: +-',num2str(Doppler),' Hz'));
disp(horzcat('Received Frequency: ',num2str(fp),' MHz'));



